/* 
 * Glippy
 * Copyright © 2010 Wojciech Kowalczyk
 * The program is distributed under the terms of the GNU General Public License Version 3.
 * See LICENCE for details.
 */
namespace Glippy
{
	/// <summary>
	/// Glippy's GConf _Client. Creates default keys, writes and reads them.
	/// </summary>		
	public class GlippyConfig // TODO rewrite, current code sucks		
	{				
		private GConf.Client _Client;	
		private static string GCONF_APP_PATH = "/apps/glippy";		
		#region Basic keys
		private static string CLIPBOARD_SIZE_KEY = GCONF_APP_PATH + "/clipboard_size";
		private static string ITEM_LENGTH_KEY = GCONF_APP_PATH + "/item_length";
		private static string REMOVE_NEW_LINES_KEY = GCONF_APP_PATH + "/remove_new_lines";
		private static string COPY_TO_MOUSE_CLIPBOARD_KEY = GCONF_APP_PATH + "/copy_to_mouse_clipboard";
		private static string SAVE_CLIPBOARD_TO_FILE = GCONF_APP_PATH + "/save_clipboard_to_file";
		private static string ENABLE_IMAGES_CLIPBOARD = GCONF_APP_PATH + "/enable_images_clipboard";
		private static string ENABLE_CLIPBOARD = GCONF_APP_PATH + "/clipboard";
		private static string ENABLE_MOUSE_CLIPBOARD = GCONF_APP_PATH + "/mouse_clipboard";		
		private int _ClipboardSize;
		private int _ItemLength;
		private bool _RemoveNewLines;
		private bool _Synchronize;
		private bool _SaveClipboardToFile;
		private bool _EnableImagesClipboard;
		private bool _EnableClipboard;
		private bool _EnableMouseClipboard;
		#endregion
		#region Hotkeys keys
		private static string ENABLE_HOTKEYS = GCONF_APP_PATH + "/hotkeys/enable_hotkeys";
		private static string HOTKEY_MODIFIER = GCONF_APP_PATH + "/hotkeys/modifier";
		private static string HOTKEY_KEYCODE = GCONF_APP_PATH + "/hotkeys/keycode";
		private bool _EnableHotkeys;
		private int _HotkeyModifier;
		private int _HotkeyKeycode;		
		#endregion		
		
		public GlippyConfig()
		{
			// GConf _Client
			_Client = new GConf.Client();			
			OnGConfChanged(this, null);
			_Client.AddNotify(GCONF_APP_PATH, new GConf.NotifyEventHandler(OnGConfChanged));
		}
		
		#region Basic properites
		public int ClipboardSize
		{
			get { return _ClipboardSize; }
			set { _Client.Set(CLIPBOARD_SIZE_KEY, value); _ClipboardSize = value; }
		}
		
		public int ItemLength
		{
			get { return _ItemLength; }
			set { _Client.Set(ITEM_LENGTH_KEY, value); _ItemLength = value; }
		}
		
		public bool RemoveNewLines
		{
			get { return _RemoveNewLines; }
			set { _Client.Set(REMOVE_NEW_LINES_KEY, value); _RemoveNewLines = value; }
		}
		
		public bool Synchronize
		{
			get { return _Synchronize; }
			set { _Client.Set(COPY_TO_MOUSE_CLIPBOARD_KEY, value); _Synchronize = value; }
		}
		
		public bool SaveClipboardToFile
		{
			get { return _SaveClipboardToFile; }
			set { _Client.Set(SAVE_CLIPBOARD_TO_FILE, value); }
		}
		
		public bool EnableImagesClipboard
		{
			get { return _EnableImagesClipboard; }
			set { _Client.Set(ENABLE_IMAGES_CLIPBOARD, value); }
		}

		public bool EnableClipboard
		{
			get { return _EnableClipboard; }
			set { _Client.Set(ENABLE_CLIPBOARD, value); }
		}

		public bool EnableMouseClipboard
		{
			get { return _EnableMouseClipboard; }
			set { _Client.Set(ENABLE_MOUSE_CLIPBOARD, value); }
		}
		#endregion
		#region Hotkeys properites
		public bool EnableHotkeys
		{
			get { return _EnableHotkeys; }
			set { _Client.Set(ENABLE_HOTKEYS, value); }			
		}
		
		public Gdk.ModifierType HotKeyModifier
		{
			get { return (Gdk.ModifierType)_HotkeyModifier; }
			set { _Client.Set(HOTKEY_MODIFIER, (int)value); }
		}
		
		public int HotkeyKeycode
		{
			get { return _HotkeyKeycode; }
			set { _Client.Set(HOTKEY_KEYCODE, value); }
		}
		#endregion

		// Reads configuration from GConf, if failed
		private void OnGConfChanged(object sender, GConf.NotifyEventArgs args)
		{
			#region Basic readers
			try
			{
				_ClipboardSize = (int) _Client.Get (CLIPBOARD_SIZE_KEY);
			}
			catch (System.Exception ex)
			{						
				System.Console.WriteLine(ex.Message);
				_Client.Set(CLIPBOARD_SIZE_KEY, 10);
				_ClipboardSize = 10;
			}
			
			try
			{
				_ItemLength = (int) _Client.Get (ITEM_LENGTH_KEY);							
			}
			catch (System.Exception ex)
			{						
				System.Console.WriteLine(ex.Message);
				_Client.Set(ITEM_LENGTH_KEY, 40);				
				_ItemLength = 40;
			}
			
			try
			{
				_RemoveNewLines = (bool) _Client.Get (REMOVE_NEW_LINES_KEY);
			}
			catch (System.Exception ex)
			{						
				System.Console.WriteLine(ex.Message);
				_Client.Set(REMOVE_NEW_LINES_KEY, true);					
				_RemoveNewLines = true;
			}
			
			try
			{
				_Synchronize = (bool) _Client.Get (COPY_TO_MOUSE_CLIPBOARD_KEY);
			}
			catch (System.Exception ex)
			{						
				System.Console.WriteLine(ex.Message);
				_Client.Set(COPY_TO_MOUSE_CLIPBOARD_KEY, false);
				_Synchronize = false;
			}
			
			try
			{
				_SaveClipboardToFile = (bool) _Client.Get (SAVE_CLIPBOARD_TO_FILE);
			}
			catch (System.Exception ex)
			{						
				System.Console.WriteLine(ex.Message);
				_Client.Set(SAVE_CLIPBOARD_TO_FILE, false);
				_SaveClipboardToFile = false;
			}
			
			try
			{
				_EnableImagesClipboard = (bool) _Client.Get (ENABLE_IMAGES_CLIPBOARD);
			}
			catch (System.Exception ex)
			{						
				System.Console.WriteLine(ex.Message);
				_Client.Set(ENABLE_IMAGES_CLIPBOARD, false);
				_EnableImagesClipboard = false;
			}
			
			try
			{
				_EnableMouseClipboard = (bool) _Client.Get (ENABLE_MOUSE_CLIPBOARD);
			}
			catch (System.Exception ex)
			{						
				System.Console.WriteLine(ex.Message);
				_Client.Set(ENABLE_MOUSE_CLIPBOARD, false);
				_EnableMouseClipboard = false;
			}
			
			try
			{
				_EnableClipboard = (bool) _Client.Get (ENABLE_CLIPBOARD);
			}
			catch (System.Exception ex)
			{						
				System.Console.WriteLine(ex.Message);
				_Client.Set(ENABLE_CLIPBOARD, true);
				_EnableClipboard = true;
			}
			#endregion	
			#region Hotkeys readers
			try
			{
				_EnableHotkeys = (bool) _Client.Get(ENABLE_HOTKEYS);
			}
			catch (System.Exception ex)
			{
				System.Console.WriteLine(ex.Message);
				_Client.Set(ENABLE_HOTKEYS, false);
				_EnableHotkeys = false;
			}
			
			try
			{
				_HotkeyModifier = (int) _Client.Get(HOTKEY_MODIFIER);
			}
			catch (System.Exception ex)
			{
				System.Console.WriteLine(ex.Message);
				_Client.Set(HOTKEY_MODIFIER, 68);
				_HotkeyModifier = 12;
			}
			
			try
			{
				_HotkeyKeycode = (int) _Client.Get(HOTKEY_KEYCODE);
			}
			catch (System.Exception ex)
			{
				System.Console.WriteLine(ex.Message);
				_Client.Set(HOTKEY_KEYCODE, 54);
				_HotkeyKeycode = 54;
			}
			#endregion
		}						
	}
}
