/* 
 * Glippy
 * Copyright © 2010 Wojciech Kowalczyk
 * The program is distributed under the terms of the GNU General Public License Version 3.
 * See LICENCE for details.
 */
namespace Glippy
{
	using System;
	using System.Collections.Generic;
	using System.Xml;

	/// <summary>
	/// History's XML parser
	/// </summary>
	public static class GlippyHistory
	{		
		public static void ReadHistory(List<GlippyItem> StoredItems, GlippyConfig _GConfig)
		{								
			string text, path = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + "/.local/share/glippy";
			GlippyItem item;
			try
			{
				if (!System.IO.Directory.Exists(path))
					System.IO.Directory.CreateDirectory(path);
				using (XmlTextReader reader = new XmlTextReader(path + "/history.xml"))
				{
					reader.Read();
					while (reader.Read())
					{
						if (reader.Name == "text")
						{
							reader.Read();
							text = reader.Value;
							text.Replace("&lt;", "<");
							text.Replace("&gt;", ">");
							System.Text.StringBuilder label = new System.Text.StringBuilder();
							label.Append(text);			
							// Cuts part of text, because we don't want too long menu				
							if (label.Length > _GConfig.ItemLength)
							{
								label.Remove(_GConfig.ItemLength/2 - 1, label.Length - _GConfig.ItemLength);
								label.Insert(label.Length/2, "...");
							}
					
							// Removing new lines in label
							if (_GConfig.RemoveNewLines)				
								label.Replace(System.Environment.NewLine, " ");
							
							item = new GlippyItem(label.ToString(), text, null);
							StoredItems.Add(item);
							
							reader.Read();
						}
					}			
				}				
			}			
			catch (Exception ex)
			{	
				Console.WriteLine(ex.Message);
			}							
		}
		
		public static void WriteHistory(List<GlippyItem> StoredItems)
		{
			try
			{
				using (XmlTextWriter writer = new XmlTextWriter(Environment.GetFolderPath(System.Environment.SpecialFolder.Personal) + "/.local/share/glippy/history.xml", System.Text.Encoding.UTF8))
				{
					writer.Formatting = Formatting.Indented;			
					writer.WriteProcessingInstruction ("xml", "version='1.0'");
					writer.WriteStartElement("items");
					
					foreach (GlippyItem i in StoredItems)
					{									
						if (i.Image == null)
						{							
							i.Text.Replace("<", "&lt;");
							i.Text.Replace(">", "&gt;");
							writer.WriteStartElement("item");
								writer.WriteStartElement("text");							
									writer.WriteString(i.Text);
								writer.WriteEndElement();
							writer.WriteEndElement();
						}
					}
							
					writer.WriteEndElement();
					writer.Close(); 
				}	
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.Message);
			}
		}
	}
}
