/* 
 * Glippy
 * Copyright © 2010 Wojciech Kowalczyk
 * The program is distributed under the terms of the GNU General Public License Version 3.
 * See LICENCE for details.
 */	
namespace Glippy
{
	using AppIndicator;
	using Gtk;
	using System;

	/// <summary>
	/// Inherits GlippyCore: creates application indicator and attaches menu to it.
	/// Leaky due to bug in libappindicator.
	/// </summary>
	public class GlippyIndicator : GlippyCore
	{
		private ApplicationIndicator _AppIndicator;	
		private MenuItem _MenuItemEmpty;		
		private const int _DEFAULT_BUTTONS_LENGTH = 5;		
		private const string _EMPTY_STRING = "";
						
		public GlippyIndicator() : base()
		{																
			_MenuItemEmpty = new MenuItem();
			_AppIndicator = new ApplicationIndicator ("glippy", "glippy-panel", Category.Other);
			_AppIndicator.AttentionIconName = "glippy-panel";
			_AppIndicator.Menu = _Menu;
			_AppIndicator.Status = Status.Active;			
			#if DEBUG
				Console.WriteLine("Indicator running fine.");
			#endif
		}		
				
		// Change event handler, reconstructs menu
		protected override void OnChange(object sender, ChangedArgs args)
		{				
			int diff = _Menu.Children.Length - _DEFAULT_BUTTONS_LENGTH;			
			MenuItem mi;
			
			if (diff < _GClip.StoredItems.Count)
			{
				for (int i = 0; i < _GClip.StoredItems.Count - diff; i++)							
				{
					_Menu.Insert(new MenuItem(_EMPTY_STRING), 0);									
					#if DEBUG
						Console.WriteLine("Adding new item.");
					#endif
				}
			}
			else if (diff > _GClip.StoredItems.Count)
			{
				for (int i = 0; i < diff - _GClip.StoredItems.Count; i++)
				{					
					#if DEBUG
						Console.WriteLine("Removing item.");
					#endif
					mi = (MenuItem)_Menu.Children[0];					
					_Menu.Remove(mi);
					mi.Destroy();
					mi.Dispose();					
				}
			}
						
			for (int i = 0; i < _GClip.StoredItems.Count; i++)
			{
				mi = (MenuItem)_Menu.Children[i];				
				foreach (Label l in mi.Children)								
				{
					#if DEBUG
						Console.WriteLine("Setting item label as \"{0}\".", _GClip.StoredItems[i].Label);
					#endif
					l.Text = _GClip.StoredItems[i].Label;													
				}
			}								
			
			// Ulgy and leaky refresh workaround
			#if DEBUG
				Console.WriteLine("Refreshing menu.");
			#endif
			_Menu.Append(_MenuItemEmpty);
			_Menu.Remove(_MenuItemEmpty);			
			_Menu.ShowAll();
		}						
				
		// Show menu when hotkey pressed
		protected override void ShowMenu(object sender, EventArgs args)
		{			
			#if DEBUG
				Console.WriteLine("Menu showed.");
			#endif
			_Menu.Popup();
		}							
	}
}
ed.");
			#endif
			_Menu.Popup();
		}							
	}
}
