/* 
 * Glippy
 * Copyright © 2010 Wojciech Kowalczyk
 * The program is distributed under the terms of the GNU General Public License Version 3.
 * See LICENCE for details.
 */
namespace Glippy
{
	/// <summary>
	/// Settings window.
	/// </summary>
	public partial class GlippySettings : Gtk.Window
	{		
		GlippyConfig _GConfig;
		GlippyXHotkey _Hotkey;		
		public event Gtk.ChangedHandler SettingsChanged;		
		
		public GlippySettings (GlippyConfig gconfig, GlippyXHotkey hotkey) : base(Gtk.WindowType.Toplevel)
		{
			this.Build ();				
			try
			{
				this.Icon = new Gdk.Pixbuf(System.AppDomain.CurrentDomain.BaseDirectory + @"/../../share/icons/hicolor/scalable/apps/glippy.svg");
			}
			catch (System.Exception ex)
			{
				System.Console.WriteLine(ex.Message);				
			}
			
			this._GConfig = gconfig;	
			this._Hotkey = hotkey;
			this._Hotkey.KeyGrabbed += OnGrabbedKey;					
								
			// Load values from GConf
			spinbuttonClipboardSize.Value = _GConfig.ClipboardSize;
			spinbuttonItemLength.Value = _GConfig.ItemLength;
			
			checkbuttonEnableClipboard.Active = _GConfig.EnableClipboard;
			checkbuttonEnableMouseClipboard.Active = _GConfig.EnableMouseClipboard;
						
			checkbuttonSynchronize.Active = _GConfig.Synchronize;
			if (checkbuttonEnableClipboard.Active && checkbuttonEnableMouseClipboard.Active)
				checkbuttonSynchronize.Sensitive = true;
						
			checkbuttonImages.Active = _GConfig.EnableImagesClipboard;
			if (checkbuttonEnableClipboard.Active)
				checkbuttonImages.Sensitive = true;
			
			checkbuttonRemoveNewLines.Active = _GConfig.RemoveNewLines;
			checkbuttonSaveClipboardToFile.Active = _GConfig.SaveClipboardToFile;
			
			checkbuttonHotkeys.Active = _GConfig.EnableHotkeys;
			if (checkbuttonHotkeys.Active)
			{
				entryKey.Sensitive = true;
				buttonGrabKey.Sensitive = true;
				labelShowMenu.Sensitive = true;
			}					
			
			SetHotkeyString();
		}

		// When spinbutton changed, raise SettingsChanged event
		private void OnSpinButtonChanged (object sender, System.EventArgs e)
		{
			Gtk.SpinButton spin = (Gtk.SpinButton) sender;
			
			if (spin == spinbuttonClipboardSize)
				_GConfig.ClipboardSize = (int)spin.Value;
			else if (spin == spinbuttonItemLength)
				_GConfig.ItemLength = (int)spin.Value;
			if (SettingsChanged != null)
				SettingsChanged(this, null);				
		}
		
		// When checkbutton toggled, raises SettingsChanged event
		private void OnCheckButtonRemoveNewLinesToggled (object sender, System.EventArgs e)
		{
			_GConfig.RemoveNewLines = checkbuttonRemoveNewLines.Active;
			if (SettingsChanged != null)
				SettingsChanged(this, null);				
		}
		
		// When checkbutton toggled, raises SettingsChanged event
		private void OnCheckbuttonSynchronizeToggled (object sender, System.EventArgs e)
		{
			_GConfig.Synchronize = checkbuttonSynchronize.Active;
			if (SettingsChanged != null)
				SettingsChanged(this, null);			
		}
		
		// When checkbutton toggled, raises SettingsChanged event	
		private void OnCheckbuttonSaveClipboardToFileToggled (object sender, System.EventArgs e)
		{
			_GConfig.SaveClipboardToFile = checkbuttonSaveClipboardToFile.Active;
			if (SettingsChanged != null)
				SettingsChanged(this, null);				
		}
		
		// What's wrong with a little destruction?
		private void OnDeleteEvent (object o, Gtk.DeleteEventArgs args)					
		{			
			if (this.Icon != null)
				this.Icon.Dispose();			
			this.Destroy();				
		}			
		
		private void OnButtonCloseClicked (object sender, System.EventArgs e)
		{
			OnDeleteEvent(null, null);			
		}			
		
		// When checkbutton toggled, raises SettingsChanged event
		private void OnCheckbuttonImagesToggled (object sender, System.EventArgs e)
		{
			_GConfig.EnableImagesClipboard = checkbuttonImages.Active;
			if (SettingsChanged != null)
				SettingsChanged(this, null);				
		}
				
		// When checkbutton toggled, enables or disables clipboard
		private void OnCheckbuttonEnableClipboardToggled (object sender, System.EventArgs e)
		{
			_GConfig.EnableClipboard = checkbuttonEnableClipboard.Active;
			
			if (checkbuttonEnableClipboard.Active && checkbuttonEnableMouseClipboard.Active)
				checkbuttonSynchronize.Sensitive = true;			
			else
				checkbuttonSynchronize.Sensitive = false;			
			
			if (checkbuttonEnableClipboard.Active)
				checkbuttonImages.Sensitive = true;
			else
				checkbuttonImages.Sensitive = false;
		}
		
		// When checkbutton toggled, enables or disables clipboard
		private void OnCheckbuttonEnableMouseClipboardToggled (object sender, System.EventArgs e)
		{
			_GConfig.EnableMouseClipboard = checkbuttonEnableMouseClipboard.Active;
			
			if (checkbuttonEnableClipboard.Active && checkbuttonEnableMouseClipboard.Active)
				checkbuttonSynchronize.Sensitive = true;			
			else
				checkbuttonSynchronize.Sensitive = false;			
		}
		
		// When checkbutton toggled, enables or disables hotkeys
		private void OnCheckbuttonHotkeysToggled (object sender, System.EventArgs e)
		{
			_GConfig.EnableHotkeys = checkbuttonHotkeys.Active;
			if (!checkbuttonHotkeys.Active)
			{
				entryKey.Sensitive = false;
				buttonGrabKey.Sensitive = false;
				labelShowMenu.Sensitive = false;
				_Hotkey.UngrabKey();
			}
			else
			{
				entryKey.Sensitive = true;
				buttonGrabKey.Sensitive = true;
				labelShowMenu.Sensitive = true;
				_Hotkey.GrabKey();
			}
		}		
		
		// When clicked, grabs keyboard and listens for new key combination
		private void OnButtonGrabKeyClicked (object sender, System.EventArgs e)
		{			
			buttonGrabKey.Sensitive = false;
			_Hotkey.GrabKeyboard();
		}				
		
		private void OnGrabbedKey(object sender, Gtk.ChangedArgs args)
		{				
			_GConfig.HotkeyKeycode = _Hotkey.Keycode;
			_GConfig.HotKeyModifier = _Hotkey.Modifiers;						
			SetHotkeyString();
			if (args != null)
			{		
				checkbuttonHotkeys.Active = false;		
				OnCheckbuttonHotkeysToggled(this, null);
			}
			buttonGrabKey.Sensitive = true;
		}		
		// Converts key to string
		private void SetHotkeyString()
		{
			System.Text.StringBuilder text = new System.Text.StringBuilder();
			text.Append("<");
			text.Append(_Hotkey.Modifiers.ToString());
			text.Append("> ");
			text.Append(_Hotkey.Key.ToString());
			text.Replace("Mask", "");
			text.Replace("Mod4", "Super");
			text.Replace("Mod1", "Alt");
			entryKey.Text = text.ToString();
		}
	}	
}
