/* 
 * Glippy
 * Copyright © 2010 Wojciech Kowalczyk
 * The program is distributed under the terms of the GNU General Public License Version 3.
 * See LICENCE for details.
 */
namespace Glippy
{	
	/// <summary>
	/// Main class.
	/// </summary>
	class MainClass
	{		
		public static void Main (string[] args)
		{						
			GlippyCore core;			
			
			Gtk.Application.Init ();				
			Mono.Unix.Catalog.Init("glippy", System.AppDomain.CurrentDomain.BaseDirectory + @"/../../share/locale");								
			#if DEBUG
				System.Console.WriteLine("Running Glippy...");
			#endif
			
			System.Reflection.Assembly asm = System.Reflection.Assembly.GetExecutingAssembly();			
			if (args.Length > 0)
			{
				if (args[0] == "--help" || args[0] == "-h")
				{					
					System.Console.Write("Glippy ");
					System.Console.WriteLine(asm.GetName().Version.ToString());				
					System.Console.WriteLine("Simple clipboard manager for GNOME");
					System.Console.WriteLine((asm.GetCustomAttributes(typeof(System.Reflection.AssemblyCopyrightAttribute), false)[0] as System.Reflection.AssemblyCopyrightAttribute).Copyright);
					System.Console.WriteLine();
					System.Console.WriteLine("Usage:");
					System.Console.WriteLine("  --help, -h\t\t\tPrint this help.");
					System.Console.WriteLine("  --appindicator, -i\t\tEnable Canonical's application indicator, EXPERIMENTAL.");
					System.Console.WriteLine("  --no-icon, -n\t\t\tDisable icon at all.");
					System.Console.WriteLine();
					return;												
				}
				else if (args[0] == "--enable-application-indicator" || args[0] == "--appindicator" || args[0] == "-i")				
					core = new GlippyIndicator();																												
				else if (args[0] == "--no-icon" || args[0] == "-n")
					core = new GlippyCore();				
				else
					return;
			}					
			else
				core = new GlippyTray();
			
			core.HandleSignal();						
					
			Gnome.Program program = new Gnome.Program("Glippy", asm.GetName().Version.ToString(), Gnome.Modules.UI, args);			
			Gnome.Client master = Gnome.Global.MasterClient();					
			master.SaveYourself += core.OnSaveYourself;	
						
			try
			{
				SetProcessName("glippy");
			}
			catch (System.ApplicationException ex)
			{
				System.Console.WriteLine(ex.Message);
			}
								
			Gtk.Application.Run ();							
		}		
					
		public static void GetArchitecture(out string arch)
		{
			System.Diagnostics.Process p = new System.Diagnostics.Process();
			p.StartInfo.FileName = "uname";
			p.StartInfo.Arguments = "-m";
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardOutput = true;			
			p.Start();			
			arch = p.StandardOutput.ReadLine();						
		}
		
		// From Banshee: Banshee.Base/Utilities.cs
		private static void SetProcessName (string name)
		{
			if (prctl (15 /* PR_SET_NAME */, System.Text.Encoding.ASCII.GetBytes (name + "\0"), System.IntPtr.Zero, System.IntPtr.Zero, System.IntPtr.Zero) != 0)
				throw new System.ApplicationException (
				        "Error setting process name: " + Mono.Unix.Native.Stdlib.GetLastError());
		}
		
		[System.Runtime.InteropServices.DllImport("libc")]
		private static extern int prctl (int option, byte[] arg2, System.IntPtr arg3, System.IntPtr arg4, System.IntPtr arg5);					
	}			
}
